'use strict';

//const http2 = require('http2');
const midCore = require('./midcore');

class middleware extends midCore {

  async go (ctx) {
    try {
      await this.run(ctx, ctx.group);
    } catch (err) {
      if (this.debug) { console.error('--DEBUG-RESPONSE--:', err); }
      if (ctx.stream) {
        try {
          if (!ctx.stream.headersSent) {
            ctx.stream.respond({
              ':status' : '500'
            });
          }
          ctx.stream.end();
        } catch (err) {}
      }
    } finally {
      ctx.res.body = null;
      ctx.stream = null;
      ctx.service = null;
      ctx.box = null;
      ctx.exec = null;
      ctx.body = null;
      ctx.headers = null;
      ctx.rawBody = null;
      ctx.files = null;
      ctx.param = null;
      ctx = null;
    }
  }

  /**
   * 这是最终添加的请求中间件。基于洋葱模型，这个中间件最先执行，所以最后会返回响应结果。
   * */
  addFinal () {
    var fr = async (ctx, next) => {
      await next();
      
      if(!ctx.stream || ctx.stream.closed || ctx.stream.destroyed) {
        return ;
      }
      var content_type = 'text/plain; charset=utf-8';
      var datatype = typeof ctx.res.body;
      /** 如果还没有发送头部信息，则判断content-type类型，然后返回。 */
      if (!ctx.stream.headersSent && ctx.res.headers['content-type'] === undefined) {
        if (datatype === 'object') {
          ctx.res.headers['content-type'] = 'text/json; charset=utf-8';
        } else if (datatype == 'string'
          && ctx.res.body.length > 1
          && ctx.res.headers['content-type'] === undefined)
        {
          switch (ctx.res.body[0]) {
            case '{':
            case '[':
              content_type = 'text/json; charset=utf-8'; break;
            case '<':
              if (ctx.res.body[1] == '!') {
                content_type = 'text/html;charset=utf-8';
              } else {
                content_type = 'text/xml;charset=utf-8';
              }
              break;
            default:;
          }
          ctx.res.headers['content-type'] = content_type;
        }
        ctx.stream.respond(ctx.res.headers);
      }

      if (datatype === 'string' && ctx.res.body.length == 0) {
        ctx.stream.end();
      } else if (ctx.res.body instanceof Buffer || datatype === 'string') {
        ctx.stream.end(ctx.res.body, ctx.res.encoding);
      } else {
        ctx.stream.end(JSON.stringify(ctx.res.body));
      }
    };
    this.add(fr);
  }

}

module.exports = middleware;
