'use strict';

const titbit = require('../main');
const zlib = require('zlib');
const fs = require('fs');
const cluster = require('cluster');

process.chdir(__dirname);

var app = new titbit({
    //daemon: true,
    maxBody: 1500000000,
    debug: true,
    useLimit: true,
    //deny : ['192.168.3.4'],
    maxIPRequest: 1000,
    maxConn: 2350,
    peerTime: 1,
    timeout : 3000,
    socktimeout: 6000,
    cert : './rsa/localhost-cert.pem',
    key : './rsa/localhost-privkey.pem',
    http2: true,
    showLoadInfo: true,
    loadInfoType : 'text',
    globalLog: true,
    //logType: 'stdio',
    loadInfoFile: '/tmp/loadinfo.log',
    //loadInfoFile : '',
    pageNotFound: `<!DOCTYPE html>
        <html>
            <head>
                <meta charset="utf-8">
                <meta name="viewport" content="width=device-width">
                <title>titbit - not found</title>
            </head>
            <body>
                <div style="margin-top:3.8rem;text-align:center;color:#565758;">
                    <h1>404 : page not found</h1>
                </div>
            </body>
        </html>
    `,
});

async function delay(t) {
  return await new Promise((rv, rj) => {
    setTimeout(() => {
      rv();
    }, t);
  });
}

//app.secure.maxrss = 36860000*2;
//console.log(app.secure.maxmem/1024/1024)

app.service.router = app.router;

var {router} = app;


var _userAgentCache = {};

app.use(async (c, next) => {
  if (c.headers['user-agent'] === undefined) {
    c.headers['user-agent'] = 'default-agent';
  }

  let ua = c.headers['user-agent'];

  let key = `${ua}:${c.ip}`;
  
  if (_userAgentCache[key] === undefined) {
    _userAgentCache[key] = {
      time : Date.now(),
      count : 1
    };
  }
  
  let uak = _userAgentCache[key];

  let tm = Date.now();
  if (tm - uak.time > 2000) {
    uak.count = 1;
    uak.time = tm;
  }

  if (uak.count > 650) {
    c.status(429);
    c.res.body = 'too many request';
    return ;
  }

  uak.count += 1;

  await next();

});

router.options('/*', async c => {
    console.log(c.param.starPath);
    c.setHeader('Access-control-allow-origin', '*');
    c.setHeader('Access-control-allow-methods', app.router.methods);
}, 'options-check');

router.get('/', async ctx => {
    ctx.res.body = 'ok';
});

router.get('/test', async ctx => {

  await delay(10);
  console.log('ok start');

  await delay(10);

  console.log('not end');
  
  await delay(10);
  console.log('ha ha!');
  
  let tout = parseInt(Math.random() * 50);

  await delay(tout);

  await new Promise((rv, rj) => {
    setTimeout(() => {
      rv();
    }, delay);
  });

  let astr = '我是中国人\n中华民族伟大复兴！\n';
  for (let i=0; i<12; i++) {
    astr += astr;
  }

  ctx.res.body = Buffer.from(`${astr}`);
});

router.post('/p', async ctx => {
    ctx.res.body = ctx.body;
}, '@post');

router.post('/pt', async ctx => {
    ctx.res.body = ctx.body;
}, {name: 'post-test2', group: 'post'});

app.get('/html', async c => {
  c.res.body = `<!DOCTYPE html><html>
      <head>
        <meta charset="utf-8">
      </head>
      <body>
        <div style="color:#676869;font-size:125%;text-align:center;">Great</div>
      </body>
    </html>`;
});

app.use(async (ctx, next) => {
    var start_time = Date.now();
    await next();
    var end_time = Date.now();
    var timing = end_time-start_time;
    console.log(process.pid,ctx.path, `: ${timing}ms`);
});


/* app.use(async (ctx, next) => {
    console.log('middleware for POST/PUT');
    await next();
    console.log('middleware for POST/PUT -- ');
}, );

app.use(async (ctx, next) => {
    console.log('a1');
    await next();
    console.log('a1');
});

app.use(async (ctx, next) => {
    console.log('a2');
    await next();
    console.log('a2');
});

app.use(async (ctx, next) => {
    console.log('a3');
    await next();
    console.log('a3');
}, {group: 'post'}); */


app.use(async (ctx, next) => {
    console.log('checking file');
    if (!ctx.isUpload) {
      console.log('不是上传文件操作');
        return ;
    }
    if (!ctx.getFile('image')) {
        ctx.send('file not found, please upload with name "image" ');
        return ;
    }
    await next();
}, (ctx) => {
  if (ctx.name !== 'upload-image') return false;
  return true;
}, 'upload');

router.post('/upload', async c => {
  try {
    console.log(c.files);
    console.log(c.body);
    let files = c.getFile('image', -1);
    let results = [];
    let fname = '';

    for(let i=0; i<files.length; i++) {
      try {
        fname = `${process.env.HOME}/tmp/a/${c.helper.makeName(files[i].filename)}`;
        await c.moveFile(files[i], fname);
        results.push(fname);
      } catch (err) {
        console.error(err);
      }
    }
    c.res.body = results;
  } catch (err) {
      c.res.body = err.message;
  }
}, {name: 'upload-image', group: 'upload'});

app.use(async (c, next) => {
    if (c.getFile('file') === null) {
        c.res.body = 'file not found -> c.files.file';
        return ;
    }
    await next();

}, (ctx) => {
  if (ctx.name !== 'upload-file') return false;
  return true;
});

router.put('/upload', async c => {
  try {
    console.log(c.files);
    console.log(c.body);
    let files = c.getFile('file', -1);
    let results = [];
    let fname = '';

    for(let i=0; i<files.length; i++) {
      try {
        fname = `${process.env.HOME}/tmp/buffer/${c.helper.makeName(files[i].filename, `${i}`)}`;
        tmp = await c.moveFile(files[i], fname);
        results.push(fname);
        //console.log('not move');
      } catch (err) {
        console.log(err);
      }
    }
    c.res.body = results;
  } catch (err) {
      c.res.body = err.message;
  }
}, {name:'upload-file', group:'upload'});

router.get('/err', async ctx => {
    throw 'Error: test';
});


app.use(async (c, next) => {
    c.cache = true;
    c.setHeader('content-encoding', 'gzip');
    c.setHeader('content-type', 'text/plain; charset=utf-8');
    await next();
    c.resBody = await new Promise((rv, rj) => {
        zlib.gzip(c.resBody, {encoding:'utf8'}, (err, data) => {
            if (err) {rj (err);}
            rv(data);
        });
    });
}, (ctx, name) => {
  if (ctx.name !== 'gzip-test') return false;
  return true;
});

router.get('/quantum', async c => {
    let data = await new Promise((rv, rj) => {
        fs.readFile('../tmp/quantum', {encoding:'utf8'}, (err, data) => {
            if (err) { rj(err); }
            rv(data);
        });
    });

    c.res.body = data;
}, 'gzip-test');

router.get('/router', async c => {
    c.res.body = [
        c.service.router.routeTable(),
        c.service.router.group()
    ];
});

//worker中设置大量定时器，测试性能
//在设置1000个定时器，仍然可以处理请求，但是比较慢
//在设置10个以内的定时器，触发时间100ms以上，对整体的性能影响比较小。
//但是这也要看定时器中进行的任务处理耗时，如果定时器中处理的是异步则不会有太大的影响。
/*
if (cluster.isWorker) {
  let randtm = 0;
  for (let i=0; i < 100; i++) { 
    randtm = i+10;
    setInterval(() => {
      console.log(process.pid,'timeout', i, process.memoryUsage(), process.cpuUsage());
    }, randtm);
  }
}
*/

if (cluster.isWorker) {
  /*
  var _sysloadinfo = '';
  process.on('message', (msg) => {
    _sysloadinfo = msg;
    console.clear();
    console.log(msg);
  });

  setInterval(() => {
    process.send({type:'loadmsg'}, undefined, undefined, (err) => {});
  }, 1000);

  app.get('/loadinfo', async c=>{
    c.res.body = _sysloadinfo;
  });
*/

}

/* if (process.argv.indexOf('-d') > 0) {
  app.config.daemon = true;
}*/

app.post('/transmit', async c => {
  c.send('ok');
}, 'transmit');

app.get('/transmit', async c => {
  c.setHeader('content-type', 'text/html;charset=utf-8');
  c.send(await c.helper.readb('./upload.html'));
});

app.pre(async (c, next) => {
  console.log('开始设定接收数据操作');
  let fwrt = fs.createWriteStream('/tmp/streambuffer');
  let total = 0;
  c.box.dataHandle = data => {
    fwrt.write(data);
    total += data.length;
  };

  await next();

  fwrt.close();

  c.send(`ok ${total} bytes`);
  console.log(total, 'bytes');

}, (ctx) => {
  if (ctx.method !== 'POST' && ctx.method !== 'PUT') {
    return false;
  }

  if (ctx.name !== 'transmit') {
    return false;
  }

  return true;
});

app.printRoute();

app.daemon(2021, 2);
